/*  This file is part of the OPIE libraries
    Copyright (C) 2002 Robert Griebl (sandman@handhelds.org)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <sys/time.h>
#include <linux/soundcard.h>
#include <math.h>

#include <qapplication.h>

#include <qfile.h>
#include <qtextstream.h>
#include <qpe/sound.h>
#include <qpe/resource.h>
#include <qpe/config.h>
#include <qpe/qcopenvelope_qws.h>

#include "odevice.h"

#include <qwindowsystem_qws.h>

#ifndef ARRAY_SIZE
#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
#endif

// _IO and friends are only defined in kernel headers ...

#define OD_IOC(dir,type,number,size)    (( dir << 30 ) | ( type << 8 ) | ( number ) | ( size << 16 ))

#define OD_IO(type,number)              OD_IOC(0,type,number,0)
#define OD_IOW(type,number,size)        OD_IOC(1,type,number,sizeof(size))
#define OD_IOR(type,number,size)        OD_IOC(2,type,number,sizeof(size))
#define OD_IORW(type,number,size)       OD_IOC(3,type,number,sizeof(size))

using namespace Opie;

class ODeviceData {
public:
	QString m_vendorstr;
	OVendor m_vendor;

	QString m_modelstr;
	OModel m_model;

	QString m_systemstr;
	OSystem m_system;

	QString m_sysverstr;
};


class iPAQ : public ODevice {
	virtual void init ( );
};

class Zaurus : public ODevice {
	virtual void init ( );
};

class SIMpad : public ODevice {
	virtual void init ( );
};

ODevice *ODevice::inst ( )
{
	static ODevice *dev = 0;

	if ( !dev ) {
		if ( QFile::exists ( "/proc/hal/model" ))
			dev = new iPAQ ( );
		else if ( QFile::exists ( "/dev/sharp_buz" ) || QFile::exists ( "/dev/sharp_led" ))
			dev = new Zaurus ( );
		else if ( QFile::exists ( "/proc/ucb1x00" ) && QFile::exists ( "/proc/CS3" ))
			dev = new SIMpad ( );
		else
			dev = new ODevice ( );

		dev-> init ( );
	}
	return dev;
}


/**************************************************
 *
 * common
 *
 **************************************************/


ODevice::ODevice ( )
{
	d = new ODeviceData;

	d-> m_modelstr = "Unknown";
	d-> m_model = Model_Unknown;
	d-> m_vendorstr = "Unknown";
	d-> m_vendor = Vendor_Unknown;
	d-> m_systemstr = "Unknown";
	d-> m_system = System_Unknown;
	d-> m_sysverstr = "0.0";
}

void ODevice::init ( )
{
}

ODevice::~ODevice ( )
{
	delete d;
}

/**
 * This returns the vendor as string
 * @return Vendor as QString
 */
QString ODevice::vendorString ( ) const
{
	return d-> m_vendorstr;
}

/**
 * This returns the vendor as one of the values of OVendor
 * @return OVendor
 */
OVendor ODevice::vendor ( ) const
{
	return d-> m_vendor;
}

/**
 * This returns the model as a string
 * @return A string representing the model
 */
QString ODevice::modelString ( ) const
{
	return d-> m_modelstr;
}

/**
 * This does return the OModel used
 */
OModel ODevice::model ( ) const
{
	return d-> m_model;
}

/**
 * This does return the systen name
 */
QString ODevice::systemString ( ) const
{
	return d-> m_systemstr;
}

/**
 * Return System as OSystem value
 */
OSystem ODevice::system ( ) const
{
	return d-> m_system;
}

/**
 * @return the version string of the base system
 */
QString ODevice::systemVersionString ( ) const
{
	return d-> m_sysverstr;
}

/**************************************************
 *
 * iPAQ
 *
 **************************************************/

void iPAQ::init ( )
{
	d-> m_vendorstr = "HP";
	d-> m_vendor = Vendor_HP;

	QFile f ( "/proc/hal/model" );

	if ( f. open ( IO_ReadOnly )) {
		QTextStream ts ( &f );

		d-> m_modelstr = "H" + ts. readLine ( );

		if ( d-> m_modelstr == "H3100" )
			d-> m_model = Model_iPAQ_H31xx;
		else if ( d-> m_modelstr == "H3600" )
			d-> m_model = Model_iPAQ_H36xx;
		else if ( d-> m_modelstr == "H3700" )
			d-> m_model = Model_iPAQ_H37xx;
		else if ( d-> m_modelstr == "H3800" )
			d-> m_model = Model_iPAQ_H38xx;
		else if ( d-> m_modelstr == "H3900" )
			d-> m_model = Model_iPAQ_H39xx;
		else
			d-> m_model = Model_Unknown;

		f. close ( );
	}

	f. setName ( "/etc/familiar-version" );
	if ( f. open ( IO_ReadOnly )) {
		d-> m_systemstr = "Familiar";
		d-> m_system = System_Familiar;

		QTextStream ts ( &f );
		d-> m_sysverstr = ts. readLine ( ). mid ( 10 );

		f. close ( );
	} else {
	    f. setName ( "/etc/oz_version" );

                    if ( f. open ( IO_ReadOnly )) {
	    	d-> m_systemstr = "OpenEmbedded/iPaq";
    		d-> m_system = System_Familiar;

	    	QTextStream ts ( &f );
		    ts.setDevice ( &f );
	    	d-> m_sysverstr = ts. readLine ( );
		    f. close ( );
        }
    }
}

/**************************************************
 *
 * Zaurus
 *
 **************************************************/



void Zaurus::init ( )
{
	d-> m_vendorstr = "Sharp";
	d-> m_vendor = Vendor_Sharp;

	QFile f ( "/proc/filesystems" );
	QString model;

	d-> m_systemstr = "Zaurus";
	d-> m_system = System_Zaurus;

	f. setName ( "/proc/cpuinfo" );
	if ( f. open ( IO_ReadOnly ) ) {
		QTextStream ts ( &f );
		QString line;
		while( line = ts. readLine ( ) ) {
			if ( line. left ( 8 ) == "Hardware" )
				break;
		}
		int loc = line. find ( ":" );
		if ( loc != -1 )
			model = line. mid ( loc + 2 ). simplifyWhiteSpace( );
	}

	// get more accurate string
	f.setName("/proc/deviceinfo/product");
	if (f.open(IO_ReadOnly)) 
	{
		QTextStream ts (&f);
		QString line;
		line = ts.readLine();
		d->m_modelstr = "Zaurus " + line;
	}

	if ( model == "SHARP Corgi" ) {
		d-> m_model = Model_Zaurus_SLC700;
		//d-> m_modelstr = "Zaurus SL-C700";
	} else if ( model == "SHARP Poodle" ) {
		d-> m_model = Model_Zaurus_SLB600;
		//d-> m_modelstr = "Zaurus SL-B500 or SL-5600";
	} else if ( model = "Sharp-Collie" ) {
		d-> m_model = Model_Zaurus_SL5500;
		//d-> m_modelstr = "Zaurus SL-5500 or SL-5000d";
	} else {
		d-> m_model = Model_Zaurus_SL5500;
		d-> m_modelstr = "Zaurus (Model unknown)";
	}

}

/**************************************************
 *
 * SIMpad
 *
 **************************************************/

void SIMpad::init ( )
{
	d-> m_vendorstr = "SIEMENS";
	d-> m_vendor = Vendor_SIEMENS;

	QFile f ( "/proc/hal/model" );

    //TODO Implement model checking
    //FIXME For now we assume an SL4

    d-> m_modelstr = "SL4";
    d-> m_model = Model_SIMpad_SL4;

	f. setName ( "/etc/familiar-version" );
	if ( f. open ( IO_ReadOnly )) {
		d-> m_systemstr = "Familiar";
		d-> m_system = System_Familiar;

		QTextStream ts ( &f );
		d-> m_sysverstr = ts. readLine ( ). mid ( 10 );

		f. close ( );
	} else {
	    f. setName ( "/etc/oz_version" );

                    if ( f. open ( IO_ReadOnly )) {
	    	d-> m_systemstr = "OpenEmbedded/SIMpad";
    		d-> m_system = System_OpenZaurus;

	    	QTextStream ts ( &f );
		    ts.setDevice ( &f );
	    	d-> m_sysverstr = ts. readLine ( );
		    f. close ( );
        }
    }
}
